"""
Schema version configuration.

This module manages supported MCI schema versions and the default version.
Maintainers can easily add new schema versions here, and in the future,
different parser implementations can be used per schema version.
"""

# Supported schema versions
SUPPORTED_SCHEMA_VERSIONS = ["1.0"]

# Default schema version
DEFAULT_SCHEMA_VERSION = "1.0"
